from typing import Protocol
from translategram.translategram.service_libs import mtranslate


class TranslatorService(Protocol):
    """
    Defines the interface for the translator services
    """

    async def translate_str(
        self, text: str, target_language: str, source_language: str = "auto"
    ) -> str:
        """
        Translate the input string to the target language.

        :param text: The text to be translated.
        :param target_language: The target language code.
        :param source_language: The source language code. Default is 'auto'.
        :return: The translated string.
        """
        ...


class MtranslateTranslatorService:
    """
    Implements the BaseTranslatorService protocol using the mtranslate library
    """

    def __init__(self) -> None:
        """
        Initialize the `MtranslateTranslatorService` instance.

        :raises AssertionError: If the `mtranslate` package is not installed.
        """
        assert mtranslate, "`MtranslateTranslatorService` requires `mtranslate` package"
        self.service = mtranslate

    async def translate_str(
        self, text: str, target_language: str = "auto", source_language: str = "auto"
    ) -> str:
        """
        Translate the input string using the `mtranslate` library.

        :param text: The text to be translated.
        :param target_language: The target language code. Default is 'auto'.
        :param source_language: The source language code. Default is 'auto'.
        :return: The translated string.
        :raises TypeError: If `text`, `target_language`, or `source_language` is not a string.
        """
        if (
            not isinstance(text, str)
            or not isinstance(target_language, str)
            or not isinstance(source_language, str)
        ):
            raise TypeError(
                "`text`, `target_language` and `source_language` must be a string"
            )
        translated_text = self.service.translate(
            to_translate=text,
            to_language=target_language,
            from_language=source_language,
        )
        return str(translated_text)



import os
import pickle
from typing import Protocol, TypeVar, Union

T = TypeVar("T")


class Cache(Protocol):
    """
    Protocol for defining a cache system.

    Implementing classes should provide functionality to store and retrieve values from the cache.
    """

    async def store(self, key: str, value: str) -> None:
        """
        Store the value in the cache associated with the specified key.

        :param key: The key to associate the value with.
        :param value: The value to store in the cache.
        """
        ...

    async def retrieve(self, key: str) -> Union[str, None]:
        """
        Retrieve the value from the cache associated with the specified key.

        :param key: The key to retrieve the value for.
        :return: The value associated with the key, or None if the key does not exist in the cache.
        """
        ...


class PickleCache:
    """
    Cache implementation using pickle serialization.

    This cache stores data in a pickle file on disk.
    """

    def __init__(self, obj: T, filename: str = "translation.data") -> None:
        """
        Initialize the PickleCache.

        :param obj: The object to be cached.
        :param filename: The name of the pickle file to store the cache data. Default is "translation.data".
        """
        self._obj = obj
        self.pickle_file = filename
        with open(self.pickle_file, "ab") as file:
            pickle.dump(self._obj, file)

    async def store(self, key: str, value: str) -> None:
        """
        Store the value in the cache associated with the specified key.

        :param key: The key to associate the value with.
        :param value: The value to store in the cache.
        """
        setattr(self._obj, key, value)
        with open(self.pickle_file, "wb") as file:
            pickle.dump(self._obj, file)

    async def retrieve(self, key: str) -> Union[str, None]:
        """
        Retrieve the value from the cache associated with the specified key.

        :param key: The key to retrieve the value for.
        :return: The value associated with the key, or None if the key does not exist in the cache.
        """
        with open(self.pickle_file, "rb") as file:
            loaded_data = pickle.load(file)
        return (
            loaded_data.__dict__.get(key)
            if isinstance(loaded_data.__dict__.get(key), str)
            else None
        )

    def __del__(self) -> None:
        """
        Clean up the cache file when the cache object is destroyed.
        """
        if os.path.exists(self.pickle_file):
            os.remove(self.pickle_file)



from translategram.python_telegram_bot_translator.adapter import (
    PythonTelegramBotAdapter as PythonTelegramBotTranslator,
)
from translategram.translategram.translator_services import MtranslateTranslatorService
from translategram.translategram.cache import PickleCache



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="translategram",
    version="0.3.0",
    description="Translategram is a Python library for translating messages are sent by your Bot in Telegram",
    url="https://github.com/EkberHasanov/translategram",
    author="Akbar",
    author_email="hasanvakbar@gmail.com",
    license="MIT",
    install_requires=[
        "mtranslate",
    ],
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests", "tests.*", "*.tests", "*.tests.*"]),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)
